@@ -1,5 +1,5 @@
 /* Simple built-in editing commands.
-   Copyright (C) 1985, 1993 Free Software Foundation, Inc.
+   Copyright (C) 1985, 1993, 1994 Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
@@ -105,7 +105,7 @@ With positive ARG, a non-empty line at the end counts as one line\n\
     }
 
   negp = count <= 0;
-  pos = scan_buffer ('\n', pos2, count - negp, &shortage);
+  pos = scan_buffer ('\n', pos2, count - negp, &shortage, 1);
   if (shortage > 0
       && (negp
 	  || (ZV > BEGV
@@ -284,7 +284,8 @@ In Auto Fill mode, if no numeric arg, break the preceding line if it's long.")
    even if it is enabled.
 
    If this insertion is suitable for direct output (completely simple),
-   return 0.  A value of 1 indicates this *might* not have been simple.  */
+   return 0.  A value of 1 indicates this *might* not have been simple.
+   A value of 2 means this did things that call for an undo boundary.  */
 
 internal_self_insert (c1, noautofill)
      char c1;
@@ -295,9 +296,11 @@ internal_self_insert (c1, noautofill)
   Lisp_Object tem;
   register enum syntaxcode synt;
   register int c = c1;
-  Lisp_Object overwrite = current_buffer->overwrite_mode;
+  Lisp_Object overwrite;
 
-  if (!NILP (Vbefore_change_function) || !NILP (Vafter_change_function))
+  overwrite = current_buffer->overwrite_mode;
+  if (!NILP (Vbefore_change_function) || !NILP (Vafter_change_function)
+      || !NILP (Vbefore_change_functions) || !NILP (Vafter_change_functions))
     hairy = 1;
 
   if (!NILP (overwrite)
@@ -311,7 +314,7 @@ internal_self_insert (c1, noautofill)
 	  || !((current_column () + 1) % XFASTINT (current_buffer->tab_width))))
     {
       del_range (point, point + 1);
-      hairy = 1;
+      hairy = 2;
     }
   if (!NILP (current_buffer->abbrev_mode)
       && SYNTAX (c) != Sword
@@ -324,7 +327,7 @@ internal_self_insert (c1, noautofill)
 	 but if Fexpand_abbrev changed the buffer,
 	 assume it expanded something.  */
       if (MODIFF != modiff)
-	hairy = 1;
+	hairy = 2;
     }
   if ((c == ' ' || c == '\n')
       && !noautofill
@@ -332,20 +335,20 @@ internal_self_insert (c1, noautofill)
       && current_column () > XFASTINT (current_buffer->fill_column))
     {
       if (c1 != '\n')
-	insert (&c1, 1);
+	insert_and_inherit (&c1, 1);
       call0 (current_buffer->auto_fill_function);
       if (c1 == '\n')
-	insert (&c1, 1);
-      hairy = 1;
+	insert_and_inherit (&c1, 1);
+      hairy = 2;
     }
   else
-    insert (&c1, 1);
+    insert_and_inherit (&c1, 1);
   synt = SYNTAX (c);
   if ((synt == Sclose || synt == Smath)
       && !NILP (Vblink_paren_function) && INTERACTIVE)
     {
       call0 (Vblink_paren_function);
-      hairy = 1;
+      hairy = 2;
     }
   return hairy;
 }
@@ -389,7 +392,11 @@ keys_of_cmds ()
   initial_define_key (global_map, Ctl('I'), "self-insert-command");
   for (n = 040; n < 0177; n++)
     initial_define_key (global_map, n, "self-insert-command");
-  for (n = 0240; n < 0377; n++)
+#ifdef MSDOS
+  for (n = 0200; n < 0240; n++)
+    initial_define_key (global_map, n, "self-insert-command");
+#endif
+  for (n = 0240; n < 0400; n++)
     initial_define_key (global_map, n, "self-insert-command");
 
   initial_define_key (global_map, Ctl ('A'), "beginning-of-line");
