@@ -20,6 +20,7 @@ along with GNU Emacs; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 
+#include <config.h>
 #include <stdio.h>
 #include <ctype.h>
 
@@ -42,26 +43,39 @@ doprnt (buffer, bufsize, format, format_end, nargs, args)
   int cnt = 0;			/* Number of arg to gobble next */
   register char *fmt = format;	/* Pointer into format string */
   register char *bufptr = buffer; /* Pointer into output buffer.. */
+
   /* Use this for sprintf unless we need something really big.  */
   char tembuf[100];
+
   /* Size of sprintf_buffer.  */
   int size_allocated = 100;
+
   /* Buffer to use for sprintf.  Either tembuf or same as BIG_BUFFER.  */
   char *sprintf_buffer = tembuf;
+
   /* Buffer we have got with malloc.  */
   char *big_buffer = 0;
+
   register int tem;
   char *string;
-  char fmtcpy[20];
+  char fixed_buffer[20];	/* Default buffer for small formatting. */
+  char *fmtcpy;
   int minlen;
   int size;			/* Field width factor; e.g., %90d */
+  char charbuf[2];		/* Used for %c.  */
 
   if (format_end == 0)
     format_end = format + strlen (format);
 
+  if ((format_end - format + 1) < sizeof (fixed_buffer))
+    fmtcpy = fixed_buffer;
+  else
+    fmtcpy = (char *) alloca (format_end - format + 1);
+
   bufsize--;
-  while (fmt != format_end && bufsize > 0)	/* Loop until end of format
-						   string or buffer full */
+
+  /* Loop until end of format string or buffer full. */
+  while (fmt != format_end && bufsize > 0)
     {
       if (*fmt == '%')	/* Check for a '%' character */
 	{
@@ -71,16 +85,23 @@ doprnt (buffer, bufsize, format, format_end, nargs, args)
 	  /* Copy this one %-spec into fmtcpy.  */
 	  string = fmtcpy;
 	  *string++ = '%';
-	  while (string < fmtcpy + sizeof fmtcpy - 1)
+	  while (1)
 	    {
 	      *string++ = *fmt;
-	      if (! (*fmt >= '0' && *fmt <= '9') && *fmt != '-' && *fmt != ' ')
+	      if (! (*fmt >= '0' && *fmt <= '9')
+		  && *fmt != '-' && *fmt != ' '&& *fmt != '.')
 		break;
 	      fmt++;
 	    }
 	  *string = 0;
 	  /* Get an idea of how much space we might need.  */
-	  size_bound = atoi (&fmtcpy[1]) + 50;
+	  size_bound = atoi (&fmtcpy[1]);
+
+	  /* Avoid pitfall of negative "size" parameter ("%-200d"). */
+	  if (size_bound < 0)
+	    size_bound = -size_bound;
+	  size_bound += 50;
+
 	  /* Make sure we have that much.  */
 	  if (size_bound > size_allocated)
 	    {
@@ -102,7 +123,7 @@ doprnt (buffer, bufsize, format, format_end, nargs, args)
 	    case 'o':
 	    case 'x':
 	      if (cnt == nargs)
-		error ("Format string wants too many arguments");
+		error ("not enough arguments for format string");
 	      sprintf (sprintf_buffer, fmtcpy, args[cnt++]);
 	      /* Now copy into final output, truncating as nec.  */
 	      string = sprintf_buffer;
@@ -114,7 +135,7 @@ doprnt (buffer, bufsize, format, format_end, nargs, args)
 	      {
 		union { double d; char *half[2]; } u;
 		if (cnt + 1 == nargs)
-		  error ("Format string wants too many arguments");
+		  error ("not enough arguments for format string");
 		u.half[0] = args[cnt++];
 		u.half[1] = args[cnt++];
 		sprintf (sprintf_buffer, fmtcpy, u.d);
@@ -127,13 +148,14 @@ doprnt (buffer, bufsize, format, format_end, nargs, args)
 	      string[-1] = 's';
 	    case 's':
 	      if (cnt == nargs)
-		error ("Format string wants too many arguments");
+		error ("not enough arguments for format string");
 	      string = args[cnt++];
 	      if (fmtcpy[1] != 's')
 		minlen = atoi (&fmtcpy[1]);
 	      /* Copy string into final output, truncating if no room.  */
 	    doit:
 	      tem = strlen (string);
+	    doit1:
 	      if (minlen > 0)
 		{
 		  while (minlen > tem && bufsize > 0)
@@ -163,10 +185,13 @@ doprnt (buffer, bufsize, format, format_end, nargs, args)
 
 	    case 'c':
 	      if (cnt == nargs)
-		error ("Format string wants too many arguments");
-	      *bufptr++ = (int) args[cnt++];
-	      bufsize--;
-	      continue;
+		error ("not enough arguments for format string");
+	      *charbuf = (int) args[cnt++];
+	      string = charbuf;
+	      tem = 1;
+	      if (fmtcpy[1] != 'c')
+		minlen = atoi (&fmtcpy[1]);
+	      goto doit1;
 
 	    case '%':
 	      fmt--;    /* Drop thru and this % will be treated as normal */
