@@ -1,5 +1,5 @@
 /* System description file for SCO 3.2v4.
-   Copyright (C) 1993 Free Software Foundation, Inc.
+   Copyright (C) 1993, 1994 Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
@@ -22,9 +22,21 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 /* SCO is sort of like SVR3.  */
 #include "usg5-3.h"
 
+#if 0 /* Turned off rather than make the Lisp code check for this. -- rms.
+	 I am assuming that (at least most of) the tests for usg-unix-v
+	 do the right thing for sco3.2v4 also.  Things that *might* be wrong
+	 as a result of turning off these lines include the values of
+	 ange-ftp-remote-shell-file-name (now remsh)
+	 dired-chown-program (now just chown)
+	 lpr-command (now lp)
+	 nntp-buggy-select (now t)
+	 rmail-spool-directory (now /usr/mail?)
+	 and the actions of the function print-region-1.  */
+
 /* SYSTEM_TYPE should indicate the kind of system you are using.  */
 #undef SYSTEM_TYPE
 #define SYSTEM_TYPE "SCO 3.2v4"
+#endif
 
 /* SCO supports job control.  */
 #undef NOMULTIPLEJOBS
@@ -45,9 +57,6 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #define PTY_TTY_NAME_SPRINTF \
   sprintf (pty_name, "/dev/ttyp%d", i);
 
-/* SCO has bcopy, et. al.  */
-#define BSTRING
-
 /* Sockets are an option on SCO.  If you have X, you have them.
    They also exist if you have TCP, but we don't know how to test
    for that.  */
@@ -55,16 +64,37 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #define HAVE_SOCKETS
 #endif
 
+/* This is safe since we already assumed HAVE_SOCKET
+   if using X windows.  */
+#undef LIBX11_SYSTEM
+#define LIBX11_SYSTEM -lpt -lnls -lnsl_s -lc_s -lsocket
+
 #ifdef HAVE_INET_SOCKETS /* This comes from autoconf.  */
 #define HAVE_SOCKETS
 #endif
 
 #ifdef HAVE_SOCKETS
-#define LIBS_SYSTEM -lsocket
+#define LIBS_SYSTEM -lsocket -lPW
 
 /* SCO has gettimeofday in socket library */
+/* Autoconf should determine this, but for now, 
+   play safe to avoid error rather than deleting this
+   and risking the wrong result.  */
+#ifndef HAVE_GETTIMEOFDAY
 #define HAVE_GETTIMEOFDAY
 #endif
+#endif
+
+/* This enables configure to tell that we have alloca.  */
+#ifndef LIBS_SYSTEM
+#define LIBS_SYSTEM -lPW
+#endif
+
+#ifdef HAVE_X11R5
+/* configure can't get this right linking fails unless -lsocket is used.  */
+#undef HAVE_XSCREENNUMBEROFSCREEN
+#define HAVE_XSCREENNUMBEROFSCREEN
+#endif
 
 /* We don't have -loldX, and we don't need it.  */
 #define LIB_XMENU_LIB
@@ -88,3 +118,11 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #else
 #define MAIL_PROGRAM_NAME "/usr/lib/mail/execmail"
 #endif
+
+/* miano@acosta.enet.dec.com says these are needed.  */
+#define bcopy(b1,b2,len) memmove (b2, b1, len)
+#define bzero(b,len) memset (b, 0, len)
+#define bcmp(b1,b2,len) memcmp (b1, b2, len)
+
+/* Tell process_send_signal to use VSUSP instead of VSWTCH.  */
+#define PREFER_VSUSP
