@@ -1,5 +1,7 @@
 #include "usg5-4.h"
 
+#define SOLARIS2
+
 /* eggert@twinsun.com said these work in Solaris.
    Perhaps they work in all kinds of SVR4, but this is more conservative.  */
 #undef BROKEN_TIOCGETC
@@ -10,12 +12,15 @@
 
 #define POSIX
 
-/* Here is how to find X Windows.  The -R option says where
-   to find X windows at run time.  */
+/* Here is how to find X Windows.  LD_SWITCH_X_SITE_AUX gives an -R option
+   says where to find X windows at run time.  */
 #ifndef __GNUC__
-#define LD_SWITCH_SYSTEM -R/usr/openwin/lib
+#define LD_SWITCH_SYSTEM LD_SWITCH_X_SITE_AUX
 #else /* GCC */
-#define LD_SWITCH_SYSTEM -Xlinker -R/usr/openwin/lib
+/* We use ./prefix-args because we don't know whether LD_SWITCH_X_SITE_AUX
+   has anything in it.  It can be empty.
+   This works ok in src.  Luckily lib-src does not use LD_SWITCH_SYSTEM.  */
+#define LD_SWITCH_SYSTEM `./prefix-args -Xlinker LD_SWITCH_X_SITE_AUX`
 #endif /* GCC */
 
 /* Compile in non-ansi fashion to work around bugs in system header files.  */
@@ -36,3 +41,9 @@ access(2), but in the meantime, some of us need the right behavior.  */
    people keep saying that it doesn't work, and that the patch is easy
    to install.  Patch number is 100947-02.  */
 #undef SOLARIS_BROKEN_ACCESS
+
+/* We can't use Xt from X11R6 with ralloc.c, because Xt
+   includes libthread.so and that calls sbrk directly.  */
+#if defined (USE_X_TOOLKIT) && defined (HAVE_X11XTR6)
+#undef REL_ALLOC
+#endif
